-----------------------------------------------------------------------------

local L = AceLibrary("AceLocale-2.2"):new("AloftBanzai")
local banzai = AceLibrary("Banzai-1.1")

-----------------------------------------------------------------------------

L:RegisterTranslations("enUS", function() return {
	["Banzai"] = true,
	["Banzai options"] = true,
	["Enable"] = true,
	["Highlights group members that have aggro"] = true,

	["Color"] = true,
	["Sets the aggro color"] = true,
} end)

-----------------------------------------------------------------------------

local AloftBanzai = Aloft:NewModule("Banzai", "AceEvent-2.0", "AceConsole-2.0", "AceHook-2.1")
AloftBanzai.db = Aloft:AcquireDBNamespace("Banzai")
Aloft:RegisterDefaults("Banzai", "profile", {
	enable			= false,
	color			= { 1, 0, 1, 1 }
})

Aloft.Options.args.Banzai =
{
	type = 'group',
	name = L["Banzai"],
	desc = L["Banzai options"],
	disabled = function() return not Aloft:IsActive() end,
	args =
	{
		enable =
		{
			type = 'toggle',
			name = L["Enable"],
			desc = L["Highlights group members that have aggro"],
			get = function() return AloftBanzai.db.profile.enable end,
			set = function(v) AloftBanzai.db.profile.enable = v Aloft:DetermineDataSources() AloftBanzai:UpdateAll() end,
			order = 1
		},
		color =
		{
			type = 'color',
			name = L["Color"],
			desc = L["Sets the aggro color"],
			disabled = function() return not AloftBanzai.db.profile.enable end,
			get = function() return unpack(AloftBanzai.db.profile.color) end,
			set = function(r, g, b, a) AloftBanzai.db.profile.color = { r, g, b, a } AloftBanzai:UpdateAll() end,
		},
	},
}

-----------------------------------------------------------------------------

function AloftBanzai:SetAggroColor(healthBar, r, g, b, a)
	local aloftData = healthBar.aloftData
	aloftData.banzaiR, aloftData.banzaiG, aloftData.banzaiB, aloftData.banzaiA = r, g, b, a
	local color = self.db.profile.color
	self.hooks[healthBar].SetStatusBarColor(healthBar, color[1], color[2], color[3], color[4])
end

function AloftBanzai:HookHealthBar(aloftData)
	local healthBar = aloftData.healthBar
	if not self:IsHooked(healthBar, "SetStatusBarColor") then
		aloftData.banzaiR, aloftData.banzaiG, aloftData.banzaiB, aloftData.banzaiA = healthBar:GetStatusBarColor()
		self:Hook(healthBar, "SetStatusBarColor", "SetAggroColor", true)
		local color = self.db.profile.color
		self.hooks[healthBar].SetStatusBarColor(healthBar, color[1], color[2], color[3], color[4])
	end
end

function AloftBanzai:UnhookHealthBar(aloftData)
	if self:IsHooked(aloftData.healthBar, "SetStatusBarColor") then 
		self:Unhook(aloftData.healthBar, "SetStatusBarColor") 
		aloftData.healthBar:SetStatusBarColor(aloftData.banzaiR, aloftData.banzaiG, aloftData.banzaiB, aloftData.banzaiA)
	end 
end

function AloftBanzai:UpdateAll()
	for aloftData in Aloft:IterateVisibleNameplates() do
		if self.db.profile.enable then
			self:OnNameplateShow(aloftData)
		else
			self:UnhookHealthBar(aloftData)
		end
	end	
	self:RegisterEvents()
end

-----------------------------------------------------------------------------

function AloftBanzai:RegisterEvents()
	self:UnregisterAllEvents()
	
	if self.db.profile.enable and Aloft:IsDataAvailable("unitid") then
		self:RegisterEvent("Banzai_UnitGainedAggro", "OnUnitGainedAggro")
		self:RegisterEvent("Banzai_UnitLostAggro", "OnUnitLostAggro")

		self:RegisterEvent("Aloft:OnNameplateShow", "OnNameplateShow")
		self:RegisterEvent("Aloft:OnNameplateHide", "OnNameplateHide")
		self:RegisterEvent("Aloft:OnUnitidChanged", "OnUnitidChanged")
	else
		self:CancelEvent("Banzai_UnitGainedAggro")
		self:CancelEvent("Banzai_UnitLostAggro")
	
		self:CancelEvent("Aloft:OnNameplateShow")
		self:CancelEvent("Aloft:OnNameplateHide")
		self:CancelEvent("Aloft:OnUnitidChanged")
	end
end

function AloftBanzai:OnEnable()
	self:UpdateAll()
end

function AloftBanzai:OnDisable()
	self:UnregisterAllEvents()
end

function AloftBanzai:RequiresData()
	local profile = self.db.profile
	
	self:RegisterEvents()

	if profile.enable and Aloft:IsDataAvailable("unitid") then
		return "unitid"
	end
end

function AloftBanzai:OnNameplateShow(aloftData)
	if aloftData.unitid and banzai:GetUnitAggroByUnitId(aloftData.unitid)  then
		self:HookHealthBar(aloftData)
	end
end

function AloftBanzai:OnNameplateHide(aloftData)
	self:UnhookHealthBar(aloftData)
end

function AloftBanzai:OnUnitidChanged(aloftData)
	self:UnhookHealthBar(aloftData)
	self:OnNameplateShow(aloftData)
end

function AloftBanzai:OnUnitLostAggro(unitid)
	local aloftData = Aloft:GetUnitNameplate(unitid)
	if aloftData then
		self:UnhookHealthBar(aloftData)
	end
end

function AloftBanzai:OnUnitGainedAggro(unitid)
	local aloftData = Aloft:GetUnitNameplate(unitid)
	if aloftData then
		self:OnNameplateShow(aloftData)
	end
end

-----------------------------------------------------------------------------
